module Jira
  class CLI < Thor

    desc "describe <ticket>", "Describes the input ticket"
    def describe(ticket)
      Command::Describe.new(ticket).run
    end

  end

  module Command
    class Describe < Base

      attr_accessor :ticket

      def initialize(ticket)
        self.ticket = ticket
      end

      def run
        return if json.empty?
        return unless errors.empty?
        render_table(header, [row])
      end

      def header
        [ 'Ticket', 'Assignee', 'Status', 'Summary' ]
      end

      def row
        [ ticket, assignee, status, summary ]
      end

      def errors
        @errors ||= (json['errorMessages'] || []).join('. ')
      end

      def assignee
        (fields['assignee'] || {})['name'] || 'Unassigned'
      end

      def status
        (fields['status'] || {})['name'] || 'Unknown'
      end

      def summary
        truncate(json['fields']['summary'], 45)
      end

      def fields
        json['fields'] || {}
      end

      def json
        @json ||= api.get "issue/#{ticket}"
      end

    end
  end
end
